include(i386impl.m4)

; see ~/info/BIOS-API.nfo

;;;;;;;;;;;;;;;;;;;;;;;; Version identification ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({API_MAGIC},{MAGIC_PC_API})
define({API_NAME},{"OSLm i386 BIOS API"})


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Constants ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({APISEG},	0x0000)		; Where the API is originally loaded
define({APIOFS},	PAGESIZE)	; The corresponding offset in PM.
define({IRQ_INT_BASE},	0x90)		; IRQ's are mapped at INT 90h
define({IDT_SIZE},	0x100)		; Is that enough ?
define({PM_STACK_SIZE},	0x400)		; 1024 entries
define({GDT_SIZE},	0x0200)		; Is that enough ?
define({COMMAND_LINE},	0x800)		; Where to put the command line


;;;;;;;;;;;;;;;;;;;;;;;;; Standard PM Selectors ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Name			sel.	base lim.	description
NULLSEG		=	0x0000	;0	0	invalid null segment
TEMPSEG		=	0x0008	;?	?	temporary
ALLMEMCS	=	0x0010	;0	4G	all memory (code)
ALLMEMDS	=	0x0018	;0	4G	all memory (data)
RMPARTCS	=	0x0020	;?	?	RM Part of the API (code)
RMPARTDS	=	0x0028	;?	?	RM Part of the API (data)
PMPARTCS	=	0x0030	;?	4G	PM Part of the API (code)
PMPARTDS	=	0x0038	;?	4G	PM Part of the API (data)
KL0CS		=	0x0040	;1G	4G	Kernel PL0 Code Segment
KL0DS		=	0x0048	;1G	4G	Kernel PL0 Data Segment
KL1CS		=	0x0051	;1G	4G	Kernel PL1 Code Segment
KL1DS		=	0x0059	;1G	4G	Kernel PL1 Data Segment
KL2CS		=	0x0062	;1G	4G	Kernel PL2 Code Segment
KL2DS		=	0x006A	;1G	4G	Kernel PL2 Data Segment
KL3CS		=	0x0073	;1G	4G	Kernel PL3 Code Segment
KL3DS		=	0x007B	;1G	4G	Kernel PL3 Data Segment
KL0CG		=	0x0080	;?	?	Kernel PL0 Call Gate
KL1CG		=	0x0089	;?	?	Kernel PL1 Call Gate
KL2CG		=	0x0092	;?	?	Kernel PL2 Call Gate
KL3CG		=	0x009B	;?	?	Kernel PL3 Call Gate
KTSS		=	0x00A0	;?	?	Kernel TSS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; segmenting ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; .text is rmpart
; .data is pmpart
; .bss is pmpart udata
; after all, everything is .text, so as86 is not confused

define({__rminitcode__},
	{indiversion(0,{
	 ;.text
	 use16
	 $*})})
define({__rminitdata__},
	{indiversion(1,{
	 ;.text
	 $*})})
define({__rmcode__},
	{indiversion(2,{
	 ;.text
	 use16
	 $*})})
define({__rmdata__},
	{indiversion(3,{
	 ;.text
	 $*})})
define({__rmstringseg__},
	{indiversion(4,{
	 ;.text
	 $*})})
define({__rmudata__},
	{indiversion(5,{
	 ;.text
	 $*})})
define({__pminitcode__},
	{indiversion(6,{
	 ;.data
	 use32
	 $*})})
define({__pminitdata__},
	{indiversion(7,{
	 ;.data
	 $*})})
define({__pmcode__},
	{indiversion(8,{
	 ;.data
	 use32
	 $*})})
define({__pmdata__},
	{indiversion(9,{
	 ;.data
	 $*})})
define({__pmstringseg__},
	{indiversion(10,{
	 ;.data
	 $*})})
define({__pmudata__},
	{indiversion(11,{
	 ;.bss
	 $*})})


define({pmaddr},	{pmofs+$1})
define({rmaddr},	{rmofs+$1})
define({initaddr},	{$1})


;;;;;;;;;;;;;;;;;;;;;;;;;; Initialize Segments ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

define({__begin__},{
__pmcode__({pmcode_beg:})
__pmdata__({pmdata_beg:})
__pmudata__({pmudata_beg:
   newheap({u},pmudata_beg,0)})
__pminitcode__({pminitcode_beg:})
__pminitdata__({pminitdata_beg:})
__rmcode__({rmcode_beg:})
__rmdata__({rmdata_beg:})
__rmudata__({rmudata_beg:
   newheap({ru},rmudata_beg,0)})
__rminitcode__({rminitcode_beg:})
__rminitdata__({rminitdata_beg:})
__rmstringseg__({rmstringseg_beg:})
__pmstringseg__({pmstringseg_beg:})

})

define({rminit},{
	define({__codeseg__},{__rminitcode__($}{@)})
	define({__dataseg__},{__rminitdata__($}{@)})
	define({__udataseg__},{__rminitdata__($}{@)})
	define({__stringseg__},{__rminitdata__($}{@)})
	define({stringaddr},{rmaddr({$1})})
})
define({pminit},{
	define({__codeseg__},{__pminitcode__($}{@)})
	define({__dataseg__},{__pminitdata__($}{@)})
	define({__udataseg__},{__pminitdata__($}{@)})
	define({__stringseg__},{__pminitdata__($}{@)})
	define({stringaddr},{pmaddr({$1})})
})
define({pmpart},{
	define({__codeseg__},{__pmcode__($}{@)})
	define({__dataseg__},{__pmdata__($}{@)})
	define({__udataseg__},{__pmudata__($}{@)})
	define({__stringseg__},{__pmstringseg__($}{@)})
	define({stringaddr},{pmaddr({$1})})
})
define({rmpart},{
	define({__codeseg__},{__rmcode__($}{@)})
	define({__dataseg__},{__rmdata__($}{@)})
	define({__udataseg__},{__rmudata__($}{@)})
	define({__stringseg__},{__rmstringseg__($}{@)})
	define({stringaddr},{rmaddr({$1})})
})


define({__end__},{
__pmcode__(
{.align 4
 pmcode_end:
})
__pmdata__(
{.align 4
 pmdata_end:
})
__pmudata__(
{.align 4
 pmudata_end = alallot(u,4,0)
})
__pminitcode__(
{.align 4
 pminitcode_end:
})
__pminitdata__(
{.align 4
 pminitdata_end:
})
__rmcode__(
{.align 4
 rmcode_end:
})
__rmdata__(
{.align 4
 rmdata_end:
})
__rmudata__(
{rmudata_end = alallot(ru,4,0)
})
__rminitcode__(
{.align 4
 rminitcode_end:
})
__rminitdata__(
{.align 4
 rminitdata_end:
})
__rmstringseg__(
{.align 4
 rmstringseg_end:
})
__pmstringseg__(
{.align 4
 pmstringseg_end:
})
;undiverting
undivert

rminitcode_len	= rminitcode_end - rminitcode_beg
rminitdata_len	= rminitdata_end - rminitdata_beg
rmcode_len	= rmcode_end - rmcode_beg
rmdata_len	= rmdata_end - rmdata_beg
rmudata_len	= rmudata_end - rmudata_beg
rmstringseg_len	= rmstringseg_end - rmstringseg_beg

pminitcode_len	= pminitcode_end - pminitcode_beg
pminitdata_len	= pminitdata_end - pminitdata_beg
pmcode_len	= pmcode_end - pmcode_beg
pmdata_len	= pmdata_end - pmdata_beg
pmudata_len	= pmudata_end - pmudata_beg
pmstringseg_len	= pmstringseg_end - pmstringseg_beg

pmpart_beg	= pmcode_beg
pmpart_end	= pmudata_end
pmpart_len	= pmpart_end - pmpart_beg
pmpart_actbeg	= pminitcode_beg
pmpart_actend	= pmstringseg_end
pmpart_actlen	= pmpart_actend - pmpart_actbeg

rmpart_beg	= rmcode_beg
rmpart_end	= rmudata_end
rmpart_len	= rmpart_end - rmpart_beg
rmpart_actbeg	= rminitcode_beg
rmpart_actend	= rmstringseg_end
rmpart_actlen	= rmpart_actend - rmpart_actbeg

pmofs	= -pmpart_end
rmofs	= 0x10000-rmpart_end


})


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Actual code ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
define({DEBUG})
define({__DEBUG__})


.org APIOFS

__OUT__({beginning}__nl__)
__begin__

__OUT__({including header.m4}__nl__)
include({header.m4})

__OUT__({including head.S}__nl__)
include({head.S})

__OUT__({including rmpart.S}__nl__)
include({rmpart.S})

__OUT__({including pmpart.S}__nl__)
include({pmpart.S})

__OUT__({including vpi386.m4}__nl__)
include({vpi386.m4})

__OUT__({end reached}__nl__)
__end__
